#


## DrQv2
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L15)
```python 
DrQv2(
   observation_space: Union[gym.Space, DictConfig], action_space: Union[gym.Space,
   DictConfig], device: str, feature_dim: int, lr: float = 0.0001, eps: float = 1e-08,
   hidden_dim: int = 1024, critic_target_tau: float = 0.01, update_every_steps: int = 2,
   network_init_method: str = 'orthogonal'
)
```


---
Data Regularized-Q v2 (DrQ-v2).
When 'augmentation' module is deprecated, this agent will transform into
    Deep Deterministic Policy Gradient (DDPG) agent.
Based on: https://github.com/facebookresearch/drqv2/blob/main/drqv2.py


**Args**

* **observation_space** (Space or DictConfig) : The observation space of environment. When invoked by Hydra,
    'observation_space' is a 'DictConfig' like {"shape": observation_space.shape, }.
* **action_space** (Space or DictConfig) : The action space of environment. When invoked by Hydra,
    'action_space' is a 'DictConfig' like
    {"shape": action_space.shape, "n": action_space.n, "type": "Discrete", "range": [0, n - 1]} or
    {"shape": action_space.shape, "type": "Box", "range": [action_space.low[0], action_space.high[0]]}.
* **device** (str) : Device (cpu, cuda, ...) on which the code should be run.
* **feature_dim** (int) : Number of features extracted by the encoder.
* **lr** (float) : The learning rate.
* **eps** (float) : Term added to the denominator to improve numerical stability.
* **hidden_dim** (int) : The size of the hidden layers.
* **critic_target_tau**  : The critic Q-function soft-update rate.
* **update_every_steps** (int) : The agent update frequency.
* **network_init_method** (str) : Network initialization method name.



**Returns**

DrQv2 agent instance.


**Methods:**


### .train
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L74)
```python
.train(
   training: bool = True
)
```

---
Set the train mode.


**Args**

* **training** (bool) : True (training) or False (testing).


**Returns**

None.

### .integrate
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L89)
```python
.integrate(
   **kwargs
)
```

---
Integrate agent and other modules (encoder, reward, ...) together

### .act
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L113)
```python
.act(
   obs: th.Tensor, training: bool = True, step: int = 0
)
```

---
Sample actions based on observations.


**Args**

* **obs** (Tensor) : Observations.
* **training** (bool) : training mode, True or False.
* **step** (int) : Global training step.


**Returns**

Sampled actions.

### .update
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L134)
```python
.update(
   replay_storage, step: int = 0
)
```

---
Update the agent.


**Args**

* **replay_storage** (Storage) : Hsuanwu replay storage.
* **step** (int) : Global training step.


**Returns**

Training metrics such as actor loss, critic_loss, etc.

### .update_critic
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L188)
```python
.update_critic(
   obs: th.Tensor, action: th.Tensor, reward: th.Tensor, discount: th.Tensor,
   next_obs: th.Tensor, step: int
)
```

---
Update the critic network.


**Args**

* **obs** (Tensor) : Observations.
* **action** (Tensor) : Actions.
* **reward** (Tensor) : Rewards.
* **discount** (Tensor) : discounts.
* **next_obs** (Tensor) : Next observations.
* **step** (int) : Global training step.


**Returns**

Critic loss metrics.

### .update_actor
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L237)
```python
.update_actor(
   obs: th.Tensor, step: int
)
```

---
Update the actor network.


**Args**

* **obs** (Tensor) : Observations.
* **step** (int) : Global training step.


**Returns**

Actor loss metrics.

### .save
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L263)
```python
.save(
   path: Path
)
```

---
Save models.


**Args**

* **path** (Path) : Storage path.


**Returns**

None.

### .load
[source](https://github.com/RLE-Foundation/Hsuanwu/blob/main/hsuanwu/xploit/agent/drqv2.py/#L280)
```python
.load(
   path: str
)
```

---
Load initial parameters.


**Args**

* **path** (str) : Import path.


**Returns**

None.
